# **********************************************************************
# Copyright 2015 VMware, Inc.  All rights reserved. VMware Confidential.
# **********************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# **********************************************************************
import socket
import struct
import textwrap


def split_address(address):
    """
    Split the address into <host-or-ip> and <port> at the last colon.
    Do not convert port to an integer.
    """
    last_colon = address.rfind(":")
    if last_colon != -1:
        host_or_ip = address[:last_colon]
        port = address[last_colon + 1:]
    else:
        host_or_ip = address
        port = None
    return host_or_ip, port


def check_rbit_set(val, n):
    """
    Check whether the right most n bits are set
    """
    mask = (1 << n) - 1
    res = val & mask
    return res == mask


def get_IPV4_CIDR_prefix_from_int(net_mask_val):
    """
    Get CIDR prefix number from subnet mask integer value (network byte order)
    """
    tmp_val = net_mask_val
    for i in range(32):
        if (tmp_val & 0x1) == 1:
            valid = check_rbit_set(tmp_val, 32 - i)
            if not valid:
                raise ValueError('Invalid net mask value {0:#x}'.format(net_mask_val))
            return 32 - i
        tmp_val = tmp_val >> 1
    return 0


def get_cidr(ip_address, netmask):
    """
    Given the IP and Mask in dotted decimal return in the IP/mask bits format.
    """
    if ip_address is None or netmask is None:
        return None
    parts = netmask.split(".")
    if len(parts) != 4:
        return None
    bits = 0
    for part in parts:
        try:
            value = int(part)
            while value != 0:
                if value % 2 == 1:
                    bits += 1
                value = int(value/2)
        except ValueError:
            return None
    return "%s/%s" % (ip_address, bits)


def mac_int_to_string(address):
    """
    Given the MAC in integer return in the MAC colon format.
    """
    return ":".join(textwrap.wrap('%012x' % (address), width=2))


def ip_int_to_string(address):
    """
    Given the IP in integer return in the IP dotted format.
    """
    return socket.inet_ntoa(struct.pack("!I", address))
