# Copyright (C) 2019 VMware, Inc. All rights reserved.
# -- VMware Confidential
import sha.config

import time
from sha.contrib.action.abc_base import Action

from sha.core.channel.provider.mpa_provider import ReportMsg
from vmware.nsx.sha import sha_pb2

from sha.contrib.action.utils.check import should_report

from sha.constants import REPORT_INTERVAL, SMALLEST_REPORT_INTERVAL_IF_CHANGE, MPA_HUB
from sha.config_monitor import ConfigMonitor


class Hbreport(Action):
    _logger = sha.config.get_logger(__name__)
    pre_report_msg = None
    pre_report_time = None
    pre_reported_by_change = False  # indicate whether pre report is caused by data change

    @classmethod
    def populate_pb_msg(cls, **kwargs):
        report_msg = sha_pb2.ShaReportMsg()
        report_msg.oid.left, report_msg.oid.right = ConfigMonitor.host_id_low_high()
        status_msg = report_msg.status.add()
        status_msg.type = sha_pb2.ShaStatusMsg.ShaStatusType.Value('HYPER_BUS')
        status_msg.hyperbus_status.status = sha_pb2.Status.Value(kwargs['status'])
        status_msg.hyperbus_status.description.append(kwargs['detail'])
        return report_msg.SerializeToString()

    @classmethod
    def do(cls, hubs, *args):
        def kernel(**kwargs):
            cls._logger.debug('Hbreport - signaling, status:{}, detail:{}, profile:{}'
                              .format(kwargs['status'], kwargs['detail'], kwargs['profile']))

            pb_msg = cls.populate_pb_msg(**kwargs)
            profile = kwargs['profile']
            do_report, report_by_change = \
                should_report(cls.pre_report_msg, cls.pre_report_time, cls.pre_reported_by_change, pb_msg,
                              profile[REPORT_INTERVAL], profile[SMALLEST_REPORT_INTERVAL_IF_CHANGE])
            cls._logger.debug('Hbreport - signaling, do_report:{}, report_by_change:{}'
                              .format(do_report, report_by_change))

            if do_report:
                cls._logger.debug('Hbreport - reporting, status:{}, detail:{}'.format(kwargs['status'], kwargs['detail']))
                hubs[MPA_HUB].send(ReportMsg(b'', pb_msg, b'com.vmware.nsx.sha.ShaReportMsg'))
                cls.pre_report_time = time.monotonic()
                cls.pre_report_msg = pb_msg
                cls.pre_reported_by_change = report_by_change

        return kernel
