# Copyright (C) 2021 VMware, Inc. All rights reserved.
# -- VMware Confidential

import json
import sys
import traceback

from sha.contrib.metric.edge_base import EdgeBase
from sha.contrib.metric.utils import run_command, timeout_cache

class TlsStats(EdgeBase):

    def get_tls_stats(self):
        tls_stats_list = []

        # Get TLS stats
        # Get TLS stats via curl command
        cmd_get_stats = ['sudo', 'ip', 'netns', 'exec', 'tlr_sr', 'curl', 'http://127.0.0.1:9001/nsx/get/stats?export=1']
        try:
            output = self.run_command_with_time_check(cmd_get_stats, timeout=10)
        except Exception as e:
            trace = traceback.format_exc()
            self._logger.debug("Failed to get TLS stats: {}, {}".format(e, trace))
            output = None
        if output is None:
            tls_stats_list = None
        else:
            tls_stats_list = json.loads(output)
        self._logger.debug("TLS stats({}): {}".format(sys.getsizeof(tls_stats_list), tls_stats_list))
        return tls_stats_list

    @timeout_cache(10)
    def usage(self, *args, **kwargs):
        # Get TLS rule stats via edge backend command
        cmd_get_rule_stats = ['edge-appctl', '-t', '/var/run/vmware/edge/dpd.ctl', 'fw/show', 'tlsrulestat']
        try:
            output = self.run_command_with_time_check(cmd_get_rule_stats, timeout=10)
        except Exception as e:
            trace = traceback.format_exc()
            self._logger.debug("Failed to get TLS rule stats: {}, {}".format(e, trace))
            output = None
        if output is None:
            rule_stats = None
        else:
            rule_stats = json.loads(output)
        self._logger.debug("TLS rule stats({}): {}".format(sys.getsizeof(rule_stats), rule_stats))

        tls_stats_list = []
        try:
            tls_stats_list = self.get_tls_stats()
        except Exception as e:
            trace = traceback.format_exc()
            self._logger.debug("Failed to get TLS stats on {}: {}, {}".format(e, trace))

        return {'tls-stats-list': tls_stats_list, \
                'rule-stats': rule_stats}
