# Copyright (C) 2021 VMware, Inc. All rights reserved.
# -- VMware Confidential

profile = data_store['profile']
tmp = data_store['metrics']
port_stats = tmp['nsx.edge.edge-lr-port-stats']
#intend uplink is external from T0
intend_port_type = ['uplink', 'loopback', 'service']
# non intend uplink is router link between T1 and T0
non_intend_port_type = ['backplane', 'uplink']
ignore_port_type = ['cpu', 'blackhole', 'downlink', 'internal-routing']

lr_port_intend_entity_map = {
        'T0' : 'TIER0_INTERFACE',
        'T1' : 'TIER1_INTERFACE',
        }
lr_port_non_intend_entity_map = {
        'T0' : 'TIER0',
        'T1' : 'TIER1',
        }

def encap_stats(stats):
    full_stats = stats.get('lr_port_stats',None)
    if full_stats is None:
        return

    for key, stats in full_stats.items():
        # we are sending stats only for routers obtained from get_lr_list
        # stats['lr_type'] and stats ['mp_router_list'] are set only for these routers
        if stats.get('lr_type',None) is None:
            continue
        if stats['ptype'] in ignore_port_type:
            logger.debug("Ignoring LR_PORT_STATS for port type = {}".format(stats['ptype']))
            continue
        stats_data = {}
        # LR port stats
        if stats is not None:
            lr_port_stats = stats['stats']
            lr_ip_port_stats = stats['ip stats']
            lr_ipv6_port_stats = stats['ipv6 stats']

            # enhancing Object ID for better filtering of the stats from DB during query
            if stats['ptype'] == 'uplink':
                if stats['lr_type'] == 'T1':
                    peer_rl = stats['peer_rl']
                    t0_mp_router_id = ''
                    for _, search_stats in full_stats.items():
                        if peer_rl == search_stats['ifuid']:
                            t0_mp_router_id = search_stats['lrouter']
                            break
                    objID = 'connected_tier0' + ':' + str(t0_mp_router_id)

                elif stats['lr_type'] == 'T0':
                    objID = key

            elif stats['ptype'] in non_intend_port_type:
                    objID = stats['ptype'] + ':' + key
            else:
                objID = key

            try:
                tmp_msg = nsx_edge_metrics_stats_pb2.NsxEdgeLRPortStats(obj_id = objID,
                                rx_pkts = lr_port_stats['rx_pkts'],
                                rx_bytes = lr_port_stats['rx_bytes'],
                                rx_drops = lr_port_stats['rx_drops'],
                                tx_pkts = lr_port_stats['tx_pkts'],
                                tx_bytes = lr_port_stats['tx_bytes'],
                                tx_drops = lr_port_stats['tx_drops'],
                                rx_drop_firewall = lr_port_stats['rx_drop_firewall'],
                                tx_drop_firewall = lr_port_stats['tx_drop_firewall'],
                                rx_drop_malformed = lr_port_stats['rx_drop_malformed'],
                                rx_drop_ipv6 = lr_port_stats['rx_drop_ipv6'],
                                rx_drop_no_route = lr_port_stats['rx_drop_no_route'],
                                rx_drop_dst_unsupported = lr_port_stats['rx_drop_dst_unsupported'],
                                rx_drop_proto_unsupported = lr_port_stats['rx_drop_proto_unsupported'],
                                rx_drop_l4port_unsupported = lr_port_stats['rx_drop_l4port_unsupported'],
                                rx_drop_ttl_exceeded = lr_port_stats['rx_drop_ttl_exceeded'],
                                rx_drop_rpf_check = lr_port_stats['rx_drop_rpf_check'],
                                rx_drop_blocked = lr_port_stats['rx_drop_blocked'],
                                rx_drop_no_receiver = lr_port_stats['rx_drop_no_receiver'],
                                rx_drop_kni = lr_port_stats['rx_drop_kni'],
                                rx_drop_redirect = lr_port_stats['rx_drop_redirect'],
                                rx_drop_rate_limit = lr_port_stats['rx_drop_rate_limit'],
                                rx_frags = lr_port_stats['rx_frags'],
                                rx_frag_ok = lr_port_stats['rx_frag_ok'],
                                rx_frag_error = lr_port_stats['rx_frag_error'],
                                rx_frag_timeout = lr_port_stats['rx_frag_timeout'],
                                rx_drop_ipsec = lr_port_stats['rx_drop_ipsec'],
                                rx_drop_ipsec_nosa = lr_port_stats['rx_drop_ipsec_nosa'],
                                rx_drop_ipsec_novti = lr_port_stats['rx_drop_ipsec_novti'],
                                rx_rs = lr_port_stats['rx_rs'],
                                rx_mld = lr_port_stats['rx_mld'],
                                tx_drop_rate_limit = lr_port_stats['tx_drop_rate_limit'],
                                tx_drop_no_arp = lr_port_stats['tx_drop_no_arp'],
                                tx_drop_frag_needed = lr_port_stats['tx_drop_frag_needed'],
                                tx_drop_blocked = lr_port_stats['tx_drop_blocked'],
                                tx_drop_no_linked = lr_port_stats['tx_drop_no_linked'],
                                tx_drop_non_ip = lr_port_stats['tx_drop_non_ip'],
                                tx_drop_no_mem = lr_port_stats['tx_drop_no_mem'],
                                tx_frag_ok = lr_port_stats['tx_frag_ok'],
                                tx_frag_error = lr_port_stats['tx_frag_error'],
                                tx_drop_ipsec = lr_port_stats['tx_drop_ipsec'],
                                tx_drop_ipsec_nosa = lr_port_stats['tx_drop_ipsec_nosa'],
                                tx_drop_ipsec_novti = lr_port_stats['tx_drop_ipsec_novti'],
                                tx_drop_ipsec_pol_block = lr_port_stats['tx_drop_ipsec_pol_block'],
                                tx_drop_ipsec_pol_err = lr_port_stats['tx_drop_ipsec_pol_err'],
                                tx_drop_service_insert = lr_port_stats['tx_drop_service_insert'],
                                tx_drop_dad = lr_port_stats['tx_drop_dad'],
                                ip_rx_pkts = lr_ip_port_stats['rx_pkts'],
                                ip_rx_bytes = lr_ip_port_stats['rx_bytes'],
                                ip_rx_drops = lr_ip_port_stats['rx_drops'],
                                ip_tx_pkts = lr_ip_port_stats['tx_pkts'],
                                ip_tx_bytes = lr_ip_port_stats['tx_bytes'],
                                ip_tx_drops = lr_ip_port_stats['tx_drops'],
                                ipv6_rx_pkts = lr_ipv6_port_stats['rx_pkts'],
                                ipv6_rx_bytes = lr_ipv6_port_stats['rx_bytes'],
                                ipv6_rx_drops = lr_ipv6_port_stats['rx_drops'],
                                ipv6_tx_pkts = lr_ipv6_port_stats['tx_pkts'],
                                ipv6_tx_bytes = lr_ipv6_port_stats['tx_bytes'],
                                ipv6_tx_drops = lr_ipv6_port_stats['tx_drops'])
            except KeyError:
                logger.info("Keyerror, not sending LR_PORT_STATS to SHA for port type = {}".format(stats['ptype']))
                continue

            if stats['ptype'] == 'uplink' and stats['lr_type'] == 'T1':
                    entity_type = options_pb2.MetricsEntityType.Value(lr_port_non_intend_entity_map[stats['lr_type']])
                    entity_id = str(stats['mp_router_id'])
            else:
                if stats['ptype'] in intend_port_type:
                    entity_type = options_pb2.MetricsEntityType.Value(lr_port_intend_entity_map[stats['lr_type']])
                    entity_id = str(stats['ifuuid'])
                elif stats['ptype'] in non_intend_port_type:
                    entity_type = options_pb2.MetricsEntityType.Value(lr_port_non_intend_entity_map[stats['lr_type']])
                    entity_id = str(stats['mp_router_id'])
                else:
                    #catch all case where the port type is not listed
                    logger.debug("Not sending LR_PORT_STATS to SHA for port type = {}.".format(stats['ptype']))
                    continue

            msg = metrics_agent_rpc_pb2.MetricsAgentMessage(timestamp=int(port_stats['timestamp']),
                                                            entity = entity_type,
                                                            entity_id = entity_id)
            sub_msg = msg.lr_port.add()
            sub_msg.CopyFrom(tmp_msg)
            logger.debug("LR_PORT_STATS exported to SHA for port type = {} and stats = {}.".format(stats['ptype'], tmp_msg))
            Exportdata(data = msg, exports = ['tsdb_metrics'])
def handle_stats (stats):
    encap_stats(stats)
if port_stats and port_stats['value']:
    logger.debug("LR_PORT_stats is {}".format(port_stats['value']))
    handle_stats(port_stats['value'])
else:
    logger.info("NO LR_PORT_STATS stats for exporting.")
